@@ -1,65 +1,87 @@
-use pyo3::{types::*, Bound, PyNativeType};
+use pyo3::{types::*, Bound};
 use serde::de::{self, IntoDeserializer};
 use serde::Deserialize;
 
-use crate::error::{PythonizeError, Result};
+use crate::error::{ErrorImpl, PythonizeError, Result};
 
 /// Attempt to convert a Python object to an instance of `T`
-#[deprecated(
-    since = "0.21.0",
-    note = "will be replaced by `depythonize_bound` in a future release"
-)]
-pub fn depythonize<'de, T>(obj: &'de PyAny) -> Result<T>
+pub fn depythonize<'a, 'py, T>(obj: &'a Bound<'py, PyAny>) -> Result<T>
 where
-    T: Deserialize<'de>,
+    T: Deserialize<'a>,
 {
-    let mut depythonizer = Depythonizer::from_object_bound(obj.as_borrowed().to_owned());
-    T::deserialize(&mut depythonizer)
-}
-
-/// Attempt to convert a Python object to an instance of `T`
-pub fn depythonize_bound<'py, T>(obj: Bound<'py, PyAny>) -> Result<T>
-where
-    T: for<'a> Deserialize<'a>,
-{
-    let mut depythonizer = Depythonizer::from_object_bound(obj);
-    T::deserialize(&mut depythonizer)
+    T::deserialize(&mut Depythonizer::from_object(obj))
 }
 
 /// A structure that deserializes Python objects into Rust values
-pub struct Depythonizer<'py> {
-    input: Bound<'py, PyAny>,
+pub struct Depythonizer<'a, 'py> {
+    input: &'a Bound<'py, PyAny>,
 }
 
-impl<'py> Depythonizer<'py> {
+impl<'a, 'py> Depythonizer<'a, 'py> {
     /// Create a deserializer from a Python object
-    #[deprecated(
-        since = "0.21.0",
-        note = "will be replaced by `Depythonizer::from_object_bound` in a future version"
-    )]
-    pub fn from_object(input: &'py PyAny) -> Self {
-        Self::from_object_bound(input.as_borrowed().to_owned())
-    }
-
-    /// Create a deserializer from a Python object
-    pub fn from_object_bound(input: Bound<'py, PyAny>) -> Self {
+    pub fn from_object(input: &'a Bound<'py, PyAny>) -> Self {
         Depythonizer { input }
     }
 
-    fn sequence_access(&self, expected_len: Option<usize>) -> Result<PySequenceAccess<'py>> {
-        let seq = self.input.downcast::<PySequence>()?;
+    fn sequence_access(&self, expected_len: Option<usize>) -> Result<PySequenceAccess<'a, 'py>> {
+        let seq = self.input.cast::<PySequence>()?;
         let len = self.input.len()?;
 
         match expected_len {
             Some(expected) if expected != len => {
                 Err(PythonizeError::incorrect_sequence_length(expected, len))
             }
-            _ => Ok(PySequenceAccess::new(seq.clone(), len)),
+            _ => Ok(PySequenceAccess::new(seq, len)),
+        }
+    }
+
+    fn set_access(&self) -> Result<PySetAsSequence<'py>> {
+        match self.input.cast::<PySet>() {
+            Ok(set) => Ok(PySetAsSequence::from_set(set)),
+            Err(e) => {
+                if let Ok(f) = self.input.cast::<PyFrozenSet>() {
+                    Ok(PySetAsSequence::from_frozenset(f))
+                } else {
+                    Err(e.into())
+                }
+            }
         }
     }
 
     fn dict_access(&self) -> Result<PyMappingAccess<'py>> {
-        PyMappingAccess::new(self.input.downcast()?)
+        PyMappingAccess::new(self.input.cast()?)
+    }
+
+    fn deserialize_any_int<'de, V>(&self, int: &Bound<'_, PyInt>, visitor: V) -> Result<V::Value>
+    where
+        V: de::Visitor<'de>,
+    {
+        if let Ok(x) = int.extract::<u128>() {
+            if let Ok(x) = u8::try_from(x) {
+                visitor.visit_u8(x)
+            } else if let Ok(x) = u16::try_from(x) {
+                visitor.visit_u16(x)
+            } else if let Ok(x) = u32::try_from(x) {
+                visitor.visit_u32(x)
+            } else if let Ok(x) = u64::try_from(x) {
+                visitor.visit_u64(x)
+            } else {
+                visitor.visit_u128(x)
+            }
+        } else {
+            let x: i128 = int.extract()?;
+            if let Ok(x) = i8::try_from(x) {
+                visitor.visit_i8(x)
+            } else if let Ok(x) = i16::try_from(x) {
+                visitor.visit_i16(x)
+            } else if let Ok(x) = i32::try_from(x) {
+                visitor.visit_i32(x)
+            } else if let Ok(x) = i64::try_from(x) {
+                visitor.visit_i64(x)
+            } else {
+                visitor.visit_i128(x)
+            }
+        }
     }
 }
 
@@ -74,14 +96,14 @@ macro_rules! deserialize_type {
     };
 }
 
-impl<'a, 'py, 'de> de::Deserializer<'de> for &'a mut Depythonizer<'py> {
+impl<'de> de::Deserializer<'de> for &'_ mut Depythonizer<'_, '_> {
     type Error = PythonizeError;
 
     fn deserialize_any<V>(self, visitor: V) -> Result<V::Value>
     where
         V: de::Visitor<'de>,
     {
-        let obj = &self.input;
+        let obj = self.input;
 
         // First check for cases which are cheap to check due to pointer
         // comparison or bitflag checks
@@ -89,8 +111,8 @@ impl<'a, 'py, 'de> de::Deserializer<'de> for &'a mut Depythonizer<'py> {
             self.deserialize_unit(visitor)
         } else if obj.is_instance_of::<PyBool>() {
             self.deserialize_bool(visitor)
-        } else if obj.is_instance_of::<PyInt>() {
-            self.deserialize_i64(visitor)
+        } else if let Ok(x) = obj.cast::<PyInt>() {
+            self.deserialize_any_int(x, visitor)
         } else if obj.is_instance_of::<PyList>() || obj.is_instance_of::<PyTuple>() {
             self.deserialize_tuple(obj.len()?, visitor)
         } else if obj.is_instance_of::<PyDict>() {
@@ -99,17 +121,16 @@ impl<'a, 'py, 'de> de::Deserializer<'de> for &'a mut Depythonizer<'py> {
             self.deserialize_str(visitor)
         }
         // Continue with cases which are slower to check because they go
-        // throuh `isinstance` machinery
+        // through `isinstance` machinery
         else if obj.is_instance_of::<PyBytes>() || obj.is_instance_of::<PyByteArray>() {
             self.deserialize_bytes(visitor)
         } else if obj.is_instance_of::<PyFloat>() {
             self.deserialize_f64(visitor)
-        } else if obj.is_instance_of::<PyFrozenSet>()
-            || obj.is_instance_of::<PySet>()
-            || obj.downcast::<PySequence>().is_ok()
-        {
+        } else if obj.is_instance_of::<PyFrozenSet>() || obj.is_instance_of::<PySet>() {
+            self.deserialize_seq(visitor)
+        } else if obj.cast::<PySequence>().is_ok() {
             self.deserialize_tuple(obj.len()?, visitor)
-        } else if obj.downcast::<PyMapping>().is_ok() {
+        } else if obj.cast::<PyMapping>().is_ok() {
             self.deserialize_map(visitor)
         } else {
             Err(obj.get_type().qualname().map_or_else(
@@ -130,7 +151,7 @@ impl<'a, 'py, 'de> de::Deserializer<'de> for &'a mut Depythonizer<'py> {
     where
         V: de::Visitor<'de>,
     {
-        let s = self.input.downcast::<PyString>()?.to_cow()?;
+        let s = self.input.cast::<PyString>()?.to_cow()?;
         if s.len() != 1 {
             return Err(PythonizeError::invalid_length_char());
         }
@@ -141,10 +162,12 @@ impl<'a, 'py, 'de> de::Deserializer<'de> for &'a mut Depythonizer<'py> {
     deserialize_type!(deserialize_i16 => visit_i16);
     deserialize_type!(deserialize_i32 => visit_i32);
     deserialize_type!(deserialize_i64 => visit_i64);
+    deserialize_type!(deserialize_i128 => visit_i128);
     deserialize_type!(deserialize_u8 => visit_u8);
     deserialize_type!(deserialize_u16 => visit_u16);
     deserialize_type!(deserialize_u32 => visit_u32);
     deserialize_type!(deserialize_u64 => visit_u64);
+    deserialize_type!(deserialize_u128 => visit_u128);
     deserialize_type!(deserialize_f32 => visit_f32);
     deserialize_type!(deserialize_f64 => visit_f64);
 
@@ -152,7 +175,7 @@ impl<'a, 'py, 'de> de::Deserializer<'de> for &'a mut Depythonizer<'py> {
     where
         V: de::Visitor<'de>,
     {
-        let s = self.input.downcast::<PyString>()?;
+        let s = self.input.cast::<PyString>()?;
         visitor.visit_str(&s.to_cow()?)
     }
 
@@ -167,7 +190,7 @@ impl<'a, 'py, 'de> de::Deserializer<'de> for &'a mut Depythonizer<'py> {
     where
         V: de::Visitor<'de>,
     {
-        let b = self.input.downcast::<PyBytes>()?;
+        let b = self.input.cast::<PyBytes>()?;
         visitor.visit_bytes(b.as_bytes())
     }
 
@@ -218,7 +241,18 @@ impl<'a, 'py, 'de> de::Deserializer<'de> for &'a mut Depythonizer<'py> {
     where
         V: de::Visitor<'de>,
     {
-        visitor.visit_seq(self.sequence_access(None)?)
+        match self.sequence_access(None) {
+            Ok(seq) => visitor.visit_seq(seq),
+            Err(e) => {
+                // we allow sets to be deserialized as sequences, so try that
+                if matches!(*e.inner, ErrorImpl::UnexpectedType(_)) {
+                    if let Ok(set) = self.set_access() {
+                        return visitor.visit_seq(set);
+                    }
+                }
+                Err(e)
+            }
+        }
     }
 
     fn deserialize_tuple<V>(self, len: usize, visitor: V) -> Result<V::Value>
@@ -269,21 +303,20 @@ impl<'a, 'py, 'de> de::Deserializer<'de> for &'a mut Depythonizer<'py> {
         V: de::Visitor<'de>,
     {
         let item = &self.input;
-        if let Ok(d) = item.downcast::<PyDict>() {
-            // Get the enum variant from the dict key
-            if d.len() != 1 {
+        if let Ok(s) = item.cast::<PyString>() {
+            visitor.visit_enum(s.to_cow()?.into_deserializer())
+        } else if let Ok(m) = item.cast::<PyMapping>() {
+            // Get the enum variant from the mapping key
+            if m.len()? != 1 {
                 return Err(PythonizeError::invalid_length_enum());
             }
-            let variant = d
-                .keys()
+            let variant: Bound<PyString> = m
+                .keys()?
                 .get_item(0)?
-                .downcast_into::<PyString>()
+                .cast_into::<PyString>()
                 .map_err(|_| PythonizeError::dict_key_not_string())?;
-            let value = d.get_item(&variant)?.unwrap();
-            let mut de = Depythonizer::from_object_bound(value);
-            visitor.visit_enum(PyEnumAccess::new(&mut de, variant))
-        } else if let Ok(s) = item.downcast::<PyString>() {
-            visitor.visit_enum(s.to_cow()?.into_deserializer())
+            let value = m.get_item(&variant)?;
+            visitor.visit_enum(PyEnumAccess::new(&value, variant))
         } else {
             Err(PythonizeError::invalid_enum_type())
         }
@@ -295,7 +328,7 @@ impl<'a, 'py, 'de> de::Deserializer<'de> for &'a mut Depythonizer<'py> {
     {
         let s = self
             .input
-            .downcast::<PyString>()
+            .cast::<PyString>()
             .map_err(|_| PythonizeError::dict_key_not_string())?;
         visitor.visit_str(&s.to_cow()?)
     }
@@ -308,19 +341,19 @@ impl<'a, 'py, 'de> de::Deserializer<'de> for &'a mut Depythonizer<'py> {
     }
 }
 
-struct PySequenceAccess<'py> {
-    seq: Bound<'py, PySequence>,
+struct PySequenceAccess<'a, 'py> {
+    seq: &'a Bound<'py, PySequence>,
     index: usize,
     len: usize,
 }
 
-impl<'py> PySequenceAccess<'py> {
-    fn new(seq: Bound<'py, PySequence>, len: usize) -> Self {
+impl<'a, 'py> PySequenceAccess<'a, 'py> {
+    fn new(seq: &'a Bound<'py, PySequence>, len: usize) -> Self {
         Self { seq, index: 0, len }
     }
 }
 
-impl<'de, 'py> de::SeqAccess<'de> for PySequenceAccess<'py> {
+impl<'de> de::SeqAccess<'de> for PySequenceAccess<'_, '_> {
     type Error = PythonizeError;
 
     fn next_element_seed<T>(&mut self, seed: T) -> Result<Option<T::Value>>
@@ -328,18 +361,53 @@ impl<'de, 'py> de::SeqAccess<'de> for PySequenceAccess<'py> {
         T: de::DeserializeSeed<'de>,
     {
         if self.index < self.len {
-            let mut item_de = Depythonizer::from_object_bound(self.seq.get_item(self.index)?);
+            let item = self.seq.get_item(self.index)?;
             self.index += 1;
-            seed.deserialize(&mut item_de).map(Some)
+            seed.deserialize(&mut Depythonizer::from_object(&item))
+                .map(Some)
         } else {
             Ok(None)
         }
     }
 }
 
+struct PySetAsSequence<'py> {
+    iter: Bound<'py, PyIterator>,
+}
+
+impl<'py> PySetAsSequence<'py> {
+    fn from_set(set: &Bound<'py, PySet>) -> Self {
+        Self {
+            iter: PyIterator::from_object(set).expect("set is always iterable"),
+        }
+    }
+
+    fn from_frozenset(set: &Bound<'py, PyFrozenSet>) -> Self {
+        Self {
+            iter: PyIterator::from_object(set).expect("frozenset is always iterable"),
+        }
+    }
+}
+
+impl<'de> de::SeqAccess<'de> for PySetAsSequence<'_> {
+    type Error = PythonizeError;
+
+    fn next_element_seed<T>(&mut self, seed: T) -> Result<Option<T::Value>>
+    where
+        T: de::DeserializeSeed<'de>,
+    {
+        match self.iter.next() {
+            Some(item) => seed
+                .deserialize(&mut Depythonizer::from_object(&item?))
+                .map(Some),
+            None => Ok(None),
+        }
+    }
+}
+
 struct PyMappingAccess<'py> {
-    keys: Bound<'py, PySequence>,
-    values: Bound<'py, PySequence>,
+    keys: Bound<'py, PyList>,
+    values: Bound<'py, PyList>,
     key_idx: usize,
     val_idx: usize,
     len: usize,
@@ -360,7 +428,7 @@ impl<'py> PyMappingAccess<'py> {
     }
 }
 
-impl<'de, 'py> de::MapAccess<'de> for PyMappingAccess<'py> {
+impl<'de> de::MapAccess<'de> for PyMappingAccess<'_> {
     type Error = PythonizeError;
 
     fn next_key_seed<K>(&mut self, seed: K) -> Result<Option<K::Value>>
@@ -368,9 +436,10 @@ impl<'de, 'py> de::MapAccess<'de> for PyMappingAccess<'py> {
         K: de::DeserializeSeed<'de>,
     {
         if self.key_idx < self.len {
-            let mut item_de = Depythonizer::from_object_bound(self.keys.get_item(self.key_idx)?);
+            let item = self.keys.get_item(self.key_idx)?;
             self.key_idx += 1;
-            seed.deserialize(&mut item_de).map(Some)
+            seed.deserialize(&mut Depythonizer::from_object(&item))
+                .map(Some)
         } else {
             Ok(None)
         }
@@ -380,24 +449,27 @@ impl<'de, 'py> de::MapAccess<'de> for PyMappingAccess<'py> {
     where
         V: de::DeserializeSeed<'de>,
     {
-        let mut item_de = Depythonizer::from_object_bound(self.values.get_item(self.val_idx)?);
+        let item = self.values.get_item(self.val_idx)?;
         self.val_idx += 1;
-        seed.deserialize(&mut item_de)
+        seed.deserialize(&mut Depythonizer::from_object(&item))
     }
 }
 
 struct PyEnumAccess<'a, 'py> {
-    de: &'a mut Depythonizer<'py>,
+    de: Depythonizer<'a, 'py>,
     variant: Bound<'py, PyString>,
 }
 
 impl<'a, 'py> PyEnumAccess<'a, 'py> {
-    fn new(de: &'a mut Depythonizer<'py>, variant: Bound<'py, PyString>) -> Self {
-        Self { de, variant }
+    fn new(obj: &'a Bound<'py, PyAny>, variant: Bound<'py, PyString>) -> Self {
+        Self {
+            de: Depythonizer::from_object(obj),
+            variant,
+        }
     }
 }
 
-impl<'a, 'py, 'de> de::EnumAccess<'de> for PyEnumAccess<'a, 'py> {
+impl<'de> de::EnumAccess<'de> for PyEnumAccess<'_, '_> {
     type Error = PythonizeError;
     type Variant = Self;
 
@@ -412,7 +484,7 @@ impl<'a, 'py, 'de> de::EnumAccess<'de> for PyEnumAccess<'a, 'py> {
     }
 }
 
-impl<'a, 'py, 'de> de::VariantAccess<'de> for PyEnumAccess<'a, 'py> {
+impl<'de> de::VariantAccess<'de> for PyEnumAccess<'_, '_> {
     type Error = PythonizeError;
 
     fn unit_variant(self) -> Result<()> {
@@ -423,7 +495,7 @@ impl<'a, 'py, 'de> de::VariantAccess<'de> for PyEnumAccess<'a, 'py> {
     where
         T: de::DeserializeSeed<'de>,
     {
-        seed.deserialize(self.de)
+        seed.deserialize(&mut { self.de })
     }
 
     fn tuple_variant<V>(self, len: usize, visitor: V) -> Result<V::Value>
@@ -443,24 +515,25 @@ impl<'a, 'py, 'de> de::VariantAccess<'de> for PyEnumAccess<'a, 'py> {
 
 #[cfg(test)]
 mod test {
+    use std::ffi::CStr;
+
     use super::*;
     use crate::error::ErrorImpl;
     use maplit::hashmap;
-    use pyo3::Python;
+    use pyo3::ffi::c_str;
+    use pyo3::{IntoPyObject, Python};
     use serde_json::{json, Value as JsonValue};
 
-    fn test_de<T>(code: &str, expected: &T, expected_json: &JsonValue)
+    fn test_de<T>(code: &CStr, expected: &T, expected_json: &JsonValue)
     where
         T: de::DeserializeOwned + PartialEq + std::fmt::Debug,
     {
-        Python::with_gil(|py| {
-            let locals = PyDict::new_bound(py);
-            py.run_bound(&format!("obj = {}", code), None, Some(&locals))
-                .unwrap();
-            let obj = locals.get_item("obj").unwrap().unwrap();
-            let actual: T = depythonize_bound(obj.clone()).unwrap();
+        Python::attach(|py| {
+            let obj = py.eval(code, None, None).unwrap();
+            let actual: T = depythonize(&obj).unwrap();
             assert_eq!(&actual, expected);
-            let actual_json: JsonValue = depythonize_bound(obj).unwrap();
+
+            let actual_json: JsonValue = depythonize(&obj).unwrap();
             assert_eq!(&actual_json, expected_json);
         });
     }
@@ -472,7 +545,7 @@ mod test {
 
         let expected = Empty;
         let expected_json = json!(null);
-        let code = "None";
+        let code = c_str!("None");
         test_de(code, &expected, &expected_json);
     }
 
@@ -498,7 +571,7 @@ mod test {
             "baz": 45.23,
             "qux": true
         });
-        let code = "{'foo': 'Foo', 'bar': 8, 'baz': 45.23, 'qux': True}";
+        let code = c_str!("{'foo': 'Foo', 'bar': 8, 'baz': 45.23, 'qux': True}");
         test_de(code, &expected, &expected_json);
     }
 
@@ -510,15 +583,13 @@ mod test {
             bar: usize,
         }
 
-        let code = "{'foo': 'Foo'}";
+        let code = c_str!("{'foo': 'Foo'}");
 
-        Python::with_gil(|py| {
-            let locals = PyDict::new_bound(py);
-            py.run_bound(&format!("obj = {}", code), None, Some(&locals))
-                .unwrap();
-            let obj = locals.get_item("obj").unwrap().unwrap();
+        Python::attach(|py| {
+            let locals = PyDict::new(py);
+            let obj = py.eval(code, None, Some(&locals)).unwrap();
             assert!(matches!(
-                *depythonize_bound::<Struct>(obj).unwrap_err().inner,
+                *depythonize::<Struct>(&obj).unwrap_err().inner,
                 ErrorImpl::Message(msg) if msg == "missing field `bar`"
             ));
         })
@@ -531,7 +602,7 @@ mod test {
 
         let expected = TupleStruct("cat".to_string(), -10.05);
         let expected_json = json!(["cat", -10.05]);
-        let code = "('cat', -10.05)";
+        let code = c_str!("('cat', -10.05)");
         test_de(code, &expected, &expected_json);
     }
 
@@ -540,15 +611,13 @@ mod test {
         #[derive(Debug, Deserialize, PartialEq)]
         struct TupleStruct(String, f64);
 
-        let code = "('cat', -10.05, 'foo')";
+        let code = c_str!("('cat', -10.05, 'foo')");
 
-        Python::with_gil(|py| {
-            let locals = PyDict::new_bound(py);
-            py.run_bound(&format!("obj = {}", code), None, Some(&locals))
-                .unwrap();
-            let obj = locals.get_item("obj").unwrap().unwrap();
+        Python::attach(|py| {
+            let locals = PyDict::new(py);
+            let obj = py.eval(code, None, Some(&locals)).unwrap();
             assert!(matches!(
-                *depythonize_bound::<TupleStruct>(obj).unwrap_err().inner,
+                *depythonize::<TupleStruct>(&obj).unwrap_err().inner,
                 ErrorImpl::IncorrectSequenceLength { expected, got } if expected == 2 && got == 3
             ));
         })
@@ -561,7 +630,7 @@ mod test {
 
         let expected = TupleStruct("cat".to_string(), -10.05);
         let expected_json = json!(["cat", -10.05]);
-        let code = "['cat', -10.05]";
+        let code = c_str!("['cat', -10.05]");
         test_de(code, &expected, &expected_json);
     }
 
@@ -569,7 +638,7 @@ mod test {
     fn test_tuple() {
         let expected = ("foo".to_string(), 5);
         let expected_json = json!(["foo", 5]);
-        let code = "('foo', 5)";
+        let code = c_str!("('foo', 5)");
         test_de(code, &expected, &expected_json);
     }
 
@@ -577,7 +646,23 @@ mod test {
     fn test_tuple_from_pylist() {
         let expected = ("foo".to_string(), 5);
         let expected_json = json!(["foo", 5]);
-        let code = "['foo', 5]";
+        let code = c_str!("['foo', 5]");
+        test_de(code, &expected, &expected_json);
+    }
+
+    #[test]
+    fn test_vec_from_pyset() {
+        let expected = vec!["foo".to_string()];
+        let expected_json = json!(["foo"]);
+        let code = c_str!("{'foo'}");
+        test_de(code, &expected, &expected_json);
+    }
+
+    #[test]
+    fn test_vec_from_pyfrozenset() {
+        let expected = vec!["foo".to_string()];
+        let expected_json = json!(["foo"]);
+        let code = c_str!("frozenset({'foo'})");
         test_de(code, &expected, &expected_json);
     }
 
@@ -585,7 +670,7 @@ mod test {
     fn test_vec() {
         let expected = vec![3, 2, 1];
         let expected_json = json!([3, 2, 1]);
-        let code = "[3, 2, 1]";
+        let code = c_str!("[3, 2, 1]");
         test_de(code, &expected, &expected_json);
     }
 
@@ -593,7 +678,7 @@ mod test {
     fn test_vec_from_tuple() {
         let expected = vec![3, 2, 1];
         let expected_json = json!([3, 2, 1]);
-        let code = "(3, 2, 1)";
+        let code = c_str!("(3, 2, 1)");
         test_de(code, &expected, &expected_json);
     }
 
@@ -601,7 +686,7 @@ mod test {
     fn test_hashmap() {
         let expected = hashmap! {"foo".to_string() => 4};
         let expected_json = json!({"foo": 4 });
-        let code = "{'foo': 4}";
+        let code = c_str!("{'foo': 4}");
         test_de(code, &expected, &expected_json);
     }
 
@@ -614,7 +699,7 @@ mod test {
 
         let expected = Foo::Variant;
         let expected_json = json!("Variant");
-        let code = "'Variant'";
+        let code = c_str!("'Variant'");
         test_de(code, &expected, &expected_json);
     }
 
@@ -627,7 +712,7 @@ mod test {
 
         let expected = Foo::Tuple(12, "cat".to_string());
         let expected_json = json!({"Tuple": [12, "cat"]});
-        let code = "{'Tuple': [12, 'cat']}";
+        let code = c_str!("{'Tuple': [12, 'cat']}");
         test_de(code, &expected, &expected_json);
     }
 
@@ -640,7 +725,7 @@ mod test {
 
         let expected = Foo::NewType("cat".to_string());
         let expected_json = json!({"NewType": "cat" });
-        let code = "{'NewType': 'cat'}";
+        let code = c_str!("{'NewType': 'cat'}");
         test_de(code, &expected, &expected_json);
     }
 
@@ -656,7 +741,7 @@ mod test {
             bar: 25,
         };
         let expected_json = json!({"Struct": {"foo": "cat", "bar": 25 }});
-        let code = "{'Struct': {'foo': 'cat', 'bar': 25}}";
+        let code = c_str!("{'Struct': {'foo': 'cat', 'bar': 25}}");
         test_de(code, &expected, &expected_json);
     }
     #[test]
@@ -669,7 +754,7 @@ mod test {
 
         let expected = Foo::Tuple(12.0, 'c');
         let expected_json = json!([12.0, 'c']);
-        let code = "[12.0, 'c']";
+        let code = c_str!("[12.0, 'c']");
         test_de(code, &expected, &expected_json);
     }
 
@@ -683,7 +768,7 @@ mod test {
 
         let expected = Foo::NewType("cat".to_string());
         let expected_json = json!("cat");
-        let code = "'cat'";
+        let code = c_str!("'cat'");
         test_de(code, &expected, &expected_json);
     }
 
@@ -700,7 +785,7 @@ mod test {
             bar: [2, 5, 3, 1],
         };
         let expected_json = json!({"foo": ["a", "b", "c"], "bar": [2, 5, 3, 1]});
-        let code = "{'foo': ['a', 'b', 'c'], 'bar': [2, 5, 3, 1]}";
+        let code = c_str!("{'foo': ['a', 'b', 'c'], 'bar': [2, 5, 3, 1]}");
         test_de(code, &expected, &expected_json);
     }
 
@@ -733,7 +818,75 @@ mod test {
         };
         let expected_json =
             json!({"name": "SomeFoo", "bar": { "value": 13, "variant": { "Tuple": [-1.5, 8]}}});
-        let code = "{'name': 'SomeFoo', 'bar': {'value': 13, 'variant': {'Tuple': [-1.5, 8]}}}";
+        let code =
+            c_str!("{'name': 'SomeFoo', 'bar': {'value': 13, 'variant': {'Tuple': [-1.5, 8]}}}");
         test_de(code, &expected, &expected_json);
     }
+
+    #[test]
+    fn test_int_limits() {
+        Python::attach(|py| {
+            // serde_json::Value supports u64 and i64 as maximum sizes
+            let _: serde_json::Value = depythonize(&u8::MAX.into_pyobject(py).unwrap()).unwrap();
+            let _: serde_json::Value = depythonize(&u8::MIN.into_pyobject(py).unwrap()).unwrap();
+            let _: serde_json::Value = depythonize(&i8::MAX.into_pyobject(py).unwrap()).unwrap();
+            let _: serde_json::Value = depythonize(&i8::MIN.into_pyobject(py).unwrap()).unwrap();
+
+            let _: serde_json::Value = depythonize(&u16::MAX.into_pyobject(py).unwrap()).unwrap();
+            let _: serde_json::Value = depythonize(&u16::MIN.into_pyobject(py).unwrap()).unwrap();
+            let _: serde_json::Value = depythonize(&i16::MAX.into_pyobject(py).unwrap()).unwrap();
+            let _: serde_json::Value = depythonize(&i16::MIN.into_pyobject(py).unwrap()).unwrap();
+
+            let _: serde_json::Value = depythonize(&u32::MAX.into_pyobject(py).unwrap()).unwrap();
+            let _: serde_json::Value = depythonize(&u32::MIN.into_pyobject(py).unwrap()).unwrap();
+            let _: serde_json::Value = depythonize(&i32::MAX.into_pyobject(py).unwrap()).unwrap();
+            let _: serde_json::Value = depythonize(&i32::MIN.into_pyobject(py).unwrap()).unwrap();
+
+            let _: serde_json::Value = depythonize(&u64::MAX.into_pyobject(py).unwrap()).unwrap();
+            let _: serde_json::Value = depythonize(&u64::MIN.into_pyobject(py).unwrap()).unwrap();
+            let _: serde_json::Value = depythonize(&i64::MAX.into_pyobject(py).unwrap()).unwrap();
+            let _: serde_json::Value = depythonize(&i64::MIN.into_pyobject(py).unwrap()).unwrap();
+
+            let _: u128 = depythonize(&u128::MAX.into_pyobject(py).unwrap()).unwrap();
+            let _: i128 = depythonize(&u128::MIN.into_pyobject(py).unwrap()).unwrap();
+
+            let _: i128 = depythonize(&i128::MAX.into_pyobject(py).unwrap()).unwrap();
+            let _: i128 = depythonize(&i128::MIN.into_pyobject(py).unwrap()).unwrap();
+        });
+    }
+
+    #[test]
+    fn test_deserialize_bytes() {
+        Python::attach(|py| {
+            let obj = PyBytes::new(py, "hello".as_bytes());
+            let actual: Vec<u8> = depythonize(&obj).unwrap();
+            assert_eq!(actual, b"hello");
+        })
+    }
+
+    #[test]
+    fn test_char() {
+        let expected = 'a';
+        let expected_json = json!("a");
+        let code = c_str!("'a'");
+        test_de(code, &expected, &expected_json);
+    }
+
+    #[test]
+    fn test_unknown_type() {
+        Python::attach(|py| {
+            let obj = py
+                .import("decimal")
+                .unwrap()
+                .getattr("Decimal")
+                .unwrap()
+                .call0()
+                .unwrap();
+            let err = depythonize::<serde_json::Value>(&obj).unwrap_err();
+            assert!(matches!(
+                *err.inner,
+                ErrorImpl::UnsupportedType(name) if name == "Decimal"
+            ));
+        });
+    }
 }
