@@ -1,6 +1,7 @@
-use pyo3::{exceptions::*, DowncastError, DowncastIntoError};
-use pyo3::{PyDowncastError, PyErr};
+use pyo3::PyErr;
+use pyo3::{exceptions::*, CastError, CastIntoError};
 use serde::{de, ser};
+use std::convert::Infallible;
 use std::error;
 use std::fmt::{self, Debug, Display};
 use std::result;
@@ -72,7 +73,7 @@ pub enum ErrorImpl {
     Message(String),
     /// A Python type not supported by the deserializer
     UnsupportedType(String),
-    /// A `PyAny` object that failed to downcast to an expected Python type
+    /// A `PyAny` object that failed to cast to an expected Python type
     UnexpectedType(String),
     /// Dict keys should be strings to deserialize to struct fields
     DictKeyNotString,
@@ -136,6 +137,13 @@ impl de::Error for PythonizeError {
     }
 }
 
+/// Convert an exception raised in Python to a `PythonizeError`
+impl From<Infallible> for PythonizeError {
+    fn from(other: Infallible) -> Self {
+        match other {}
+    }
+}
+
 /// Convert an exception raised in Python to a `PythonizeError`
 impl From<PyErr> for PythonizeError {
     fn from(other: PyErr) -> Self {
@@ -145,27 +153,18 @@ impl From<PyErr> for PythonizeError {
     }
 }
 
-/// Handle errors that occur when attempting to use `PyAny::cast_as`
-impl<'a> From<PyDowncastError<'a>> for PythonizeError {
-    fn from(other: PyDowncastError) -> Self {
+/// Handle errors that occur when attempting to use `PyAny::cast`
+impl<'a, 'py> From<CastError<'a, 'py>> for PythonizeError {
+    fn from(other: CastError<'a, 'py>) -> Self {
         Self {
             inner: Box::new(ErrorImpl::UnexpectedType(other.to_string())),
         }
     }
 }
 
-/// Handle errors that occur when attempting to use `PyAny::cast_as`
-impl<'a, 'py> From<DowncastError<'a, 'py>> for PythonizeError {
-    fn from(other: DowncastError<'a, 'py>) -> Self {
-        Self {
-            inner: Box::new(ErrorImpl::UnexpectedType(other.to_string())),
-        }
-    }
-}
-
-/// Handle errors that occur when attempting to use `PyAny::cast_as`
-impl<'py> From<DowncastIntoError<'py>> for PythonizeError {
-    fn from(other: DowncastIntoError<'py>) -> Self {
+/// Handle errors that occur when attempting to use `PyAny::cast`
+impl<'py> From<CastIntoError<'py>> for PythonizeError {
+    fn from(other: CastIntoError<'py>) -> Self {
         Self {
             inner: Box::new(ErrorImpl::UnexpectedType(other.to_string())),
         }
