@@ -13,9 +13,9 @@ struct Root<T> {
     root_map: BTreeMap<String, Nested<T>>,
 }
 
-impl<'py, T> PythonizeTypes<'py> for Root<T> {
+impl<'py, T> PythonizeTypes for Root<T> {
     type Map = PyDict;
-    type NamedMap = PythonizeUnnamedMappingAdapter<'py, PyDict>;
+    type NamedMap = PythonizeUnnamedMappingAdapter<PyDict>;
     type List = PyList;
 }
 
@@ -40,7 +40,7 @@ impl Serialize for CannotSerialize {
 
 #[test]
 fn test_de_valid() {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let pyroot = PyDict::new(py);
         pyroot.set_item("root_key", "root_value").unwrap();
 
@@ -82,7 +82,7 @@ fn test_de_valid() {
 
 #[test]
 fn test_de_invalid() {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let pyroot = PyDict::new(py);
         pyroot.set_item("root_key", "root_value").unwrap();
 
@@ -100,13 +100,16 @@ fn test_de_invalid() {
         let err = serde_path_to_error::deserialize::<_, Root<String>>(de).unwrap_err();
 
         assert_eq!(err.path().to_string(), "root_map.nested_1.nested_key");
-        assert_eq!(err.to_string(), "root_map.nested_1.nested_key: unexpected type: 'int' object cannot be converted to 'PyString'");
+        assert_eq!(
+            err.to_string(),
+            "root_map.nested_1.nested_key: unexpected type: 'int' object cannot be cast as 'str'"
+        );
     })
 }
 
 #[test]
 fn test_ser_valid() {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let root = Root {
             root_key: String::from("root_value"),
             root_map: BTreeMap::from([
@@ -128,7 +131,7 @@ fn test_ser_valid() {
         let ser = pythonize::Pythonizer::<Root<String>>::from(py);
         let pyroot: Bound<'_, PyAny> = serde_path_to_error::serialize(&root, ser).unwrap();
 
-        let pyroot = pyroot.downcast::<PyDict>().unwrap();
+        let pyroot = pyroot.cast::<PyDict>().unwrap();
         assert_eq!(pyroot.len(), 2);
 
         let root_value: String = pyroot
@@ -143,7 +146,7 @@ fn test_ser_valid() {
             .get_item("root_map")
             .unwrap()
             .unwrap()
-            .downcast_into::<PyDict>()
+            .cast_into::<PyDict>()
             .unwrap();
         assert_eq!(root_map.len(), 2);
 
@@ -151,7 +154,7 @@ fn test_ser_valid() {
             .get_item("nested_0")
             .unwrap()
             .unwrap()
-            .downcast_into::<PyDict>()
+            .cast_into::<PyDict>()
             .unwrap();
         assert_eq!(nested_0.len(), 1);
         let nested_key_0: String = nested_0
@@ -166,7 +169,7 @@ fn test_ser_valid() {
             .get_item("nested_1")
             .unwrap()
             .unwrap()
-            .downcast_into::<PyDict>()
+            .cast_into::<PyDict>()
             .unwrap();
         assert_eq!(nested_1.len(), 1);
         let nested_key_1: String = nested_1
@@ -181,7 +184,7 @@ fn test_ser_valid() {
 
 #[test]
 fn test_ser_invalid() {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let root = Root {
             root_key: String::from("root_value"),
             root_map: BTreeMap::from([
